@@ -4,7 +4,7 @@ from typing import Any
 
 import numpy as np
 from scipy import stats
-from scipy.stats import beta as beta_dist
+from scipy.special import betaln, gammaln
 
 # Exported public API
 __all__ = [
@@ -13,6 +13,7 @@ __all__ = [
     "clopper_pearson_intervals",
     "cp_interval",
     "prediction_bounds",
+    "prediction_bounds_beta_binomial",
 ]
 
 
@@ -48,7 +49,7 @@ def clopper_pearson_lower(k: int, n: int, confidence: float = 0.95) -> float:
     # L = Beta^{-1}(1-confidence; k, n-k+1)
     # Note: Using (1-confidence) as the lower tail probability
     alpha = 1 - confidence
-    return float(beta_dist.ppf(alpha, k, n - k + 1))
+    return float(stats.beta.ppf(alpha, k, n - k + 1))
 
 
 def clopper_pearson_upper(k: int, n: int, confidence: float = 0.95) -> float:
@@ -82,7 +83,7 @@ def clopper_pearson_upper(k: int, n: int, confidence: float = 0.95) -> float:
         return 1.0
     # U = Beta^{-1}(confidence; k+1, n-k)
     # Note: Using confidence directly for upper tail
-    return float(beta_dist.ppf(confidence, k + 1, n - k))
+    return float(stats.beta.ppf(confidence, k + 1, n - k))
 
 
 def clopper_pearson_intervals(labels: np.ndarray, confidence: float = 0.95) -> dict[int, dict[str, Any]]:
@@ -173,7 +174,12 @@ def cp_interval(count: int, total: int, confidence: float = 0.95) -> dict[str, f
     total = int(total)
 
     if total <= 0:
-        return {"count": count, "proportion": 0.0, "lower": 0.0, "upper": 0.0}
+        return {
+            "count": count,
+            "proportion": float("nan"),
+            "lower": 0.0,
+            "upper": 1.0,
+        }
 
     p = count / total
 
@@ -225,8 +231,6 @@ def prediction_bounds_lower(k_cal: int, n_cal: int, n_test: int, confidence: flo
     For large n_test, bounds approach calibration-only bounds.
     For small n_test, bounds are wider due to additional test set sampling uncertainty.
     """
-    if k_cal == 0:
-        return 0.0
     if n_cal <= 0 or n_test <= 0:
         return 0.0
 
@@ -283,8 +287,6 @@ def prediction_bounds_upper(k_cal: int, n_cal: int, n_test: int, confidence: flo
     For large n_test, bounds approach calibration-only bounds.
     For small n_test, bounds are wider due to additional test set sampling uncertainty.
     """
-    if k_cal == n_cal:
-        return 1.0
     if n_cal <= 0 or n_test <= 0:
         return 1.0
 
@@ -306,14 +308,45 @@ def prediction_bounds_upper(k_cal: int, n_cal: int, n_test: int, confidence: flo
     return max(0.0, min(1.0, upper_bound))
 
 
-def prediction_bounds_beta_binomial(
-    k_cal: int, n_cal: int, n_test: int, confidence: float = 0.95
-) -> tuple[float, float]:
-    """Compute prediction bounds using Beta-Binomial distribution (sophisticated method).
+def _log_binomial_coef(n: int, k: int) -> float:
+    """Compute log(n choose k) = log(n!) - log(k!) - log((n-k)!)."""
+    return gammaln(n + 1) - gammaln(k + 1) - gammaln(n - k + 1)
 
-    This function uses the Beta-Binomial distribution to model the uncertainty in both
-    calibration and test set sampling. This is more accurate than the simple method
-    for small sample sizes.
+
+def _beta_binomial_pmf(k: int, n: int, alpha: float, beta: float) -> float:
+    """Beta-binomial PMF: pmf(k; n, alpha, beta) = C(n,k) * Beta(k+alpha, n-k+beta) / Beta(alpha, beta).
+
+    Parameters
+    ----------
+    k : int
+        Number of successes
+    n : int
+        Total number of trials
+    alpha : float
+        First Beta parameter
+    beta : float
+        Second Beta parameter
+
+    Returns
+    -------
+    float
+        Probability mass at k
+    """
+    return np.exp(_log_binomial_coef(n, k) + betaln(k + alpha, n - k + beta) - betaln(alpha, beta))
+
+
+def prediction_bounds_beta_binomial(
+    k_cal: int,
+    n_cal: int,
+    n_test: int,
+    confidence: float = 0.95,
+    use_jeffreys: bool = False,
+    tail: str = "equal_tailed",
+) -> tuple[float, float]:
+    """Beta-Binomial predictive interval for future empirical rate.
+
+    This function computes exact prediction bounds using the Beta-Binomial distribution,
+    which properly accounts for both calibration uncertainty and test set sampling variability.
 
     Parameters
     ----------
@@ -324,66 +357,71 @@ def prediction_bounds_beta_binomial(
     n_test : int
         Expected size of future test sets
     confidence : float, default=0.95
-        Confidence level (e.g., 0.95 for 95% prediction bounds)
+        Desired predictive mass (confidence level)
+    use_jeffreys : bool, default=False
+        If False, use uniform prior Beta(1,1), giving posterior Beta(k_cal+1, n_cal-k_cal+1).
+        If True, use Jeffreys prior Beta(1/2,1/2), giving posterior Beta(k_cal+0.5, n_cal-k_cal+0.5).
+    tail : str, default="equal_tailed"
+        Interval type:
+        - "equal_tailed": Invert predictive CDF (α/2 each side)
+        - "hpd": Shortest high posterior density predictive interval
 
     Returns
     -------
     tuple[float, float]
-        (lower_bound, upper_bound) for operational rates on future test sets
+        (lower_rate, upper_rate) for operational rates on future test sets
 
     Notes
     -----
     This method models:
-    1. True rate p ~ Beta(k_cal + 1, n_cal - k_cal + 1) (from calibration uncertainty)
-    2. Test set rate r | p ~ Binomial(n_test, p) / n_test (from test set sampling)
-    3. Marginal distribution: r ~ BetaBinomial(n_test, k_cal + 1, n_cal - k_cal + 1) / n_test
+    1. True rate p ~ Beta(alpha, beta) (posterior from calibration data)
+    2. Future successes k_test | p ~ Binomial(n_test, p)
+    3. Marginal predictive distribution: k_test ~ BetaBinomial(n_test, alpha, beta)
+    4. Return bounds on the rate k_test / n_test
 
-    This is more accurate than the simple method for small sample sizes.
+    This provides exact finite-sample prediction bounds that account for both sources
+    of uncertainty without normal approximations.
     """
-    if k_cal == 0:
-        return (0.0, 1.0)
-    if k_cal == n_cal:
-        return (0.0, 1.0)
     if n_cal <= 0 or n_test <= 0:
         return (0.0, 1.0)
 
-    # Beta parameters from calibration data (Jeffreys prior)
-    alpha = k_cal + 1
-    beta = n_cal - k_cal + 1
+    offset = 0.5 if use_jeffreys else 1.0
+    alpha = k_cal + offset
+    beta = (n_cal - k_cal) + offset
 
-    # For the Beta-Binomial approach, we need to account for both sources of uncertainty
-    # The variance of the Beta-Binomial distribution is:
-    # Var(X) = n * p * (1-p) * (1 + (n-1) * rho)
-    # where rho is the correlation parameter
+    # Predictive PMF over k_test = 0..n_test
+    ks = np.arange(n_test + 1)
+    pmf = np.array([_beta_binomial_pmf(k, n_test, alpha, beta) for k in ks])
+    pmf = pmf / pmf.sum()  # Numerical safety (normalize to ensure sum = 1)
 
-    # We'll use a more sophisticated approach that properly accounts for test set sampling
-    # by using the Beta distribution for the true rate and then accounting for test set variability
+    if tail == "equal_tailed":
+        cdf = np.cumsum(pmf)
+        alpha_tail = (1.0 - confidence) / 2.0
 
-    # Compute quantiles of the Beta distribution for the true rate
-    alpha_quantile = (1 - confidence) / 2
-    upper_quantile = 1 - alpha_quantile
+        # Lower index = smallest k with CDF >= alpha_tail
+        k_lo = int(np.searchsorted(cdf, alpha_tail))
+        # Upper index = largest k with CDF <= 1 - alpha_tail
+        k_hi = int(np.searchsorted(cdf, 1.0 - alpha_tail, side="right") - 1)
 
-    # Get the Beta distribution quantiles for the true rate
-    lower_rate = stats.beta.ppf(alpha_quantile, alpha, beta)
-    upper_rate = stats.beta.ppf(upper_quantile, alpha, beta)
+    elif tail == "hpd":
+        # Sort ks by PMF descending, then add mass until >= confidence,
+        # then take min/max k in that included set. This gives a highest-density
+        # predictive set, then we report its hull.
+        order = np.argsort(-pmf)
+        keep = []
+        mass = 0.0
+        for idx in order:
+            keep.append(idx)
+            mass += pmf[idx]
+            if mass >= confidence:
+                break
+        k_lo = ks[min(keep)]
+        k_hi = ks[max(keep)]
 
-    # Now account for test set sampling uncertainty by adding the binomial variance
-    # The total variance is: Var(p) + Var(r|p) = Var(p) + p*(1-p)/n_test
-    # We'll use a conservative approach by expanding the bounds
+    else:
+        raise ValueError("tail must be 'equal_tailed' or 'hpd'")
 
-    # Estimate the additional uncertainty from test set sampling
-    # Use the mean rate as an approximation
-    mean_rate = alpha / (alpha + beta)
-    test_set_se = np.sqrt(mean_rate * (1 - mean_rate) / n_test)
-
-    # Expand bounds to account for test set sampling uncertainty
-    z_score = stats.norm.ppf(1 - alpha_quantile)
-    margin = z_score * test_set_se
-
-    lower_bound = max(0.0, lower_rate - margin)
-    upper_bound = min(1.0, upper_rate + margin)
-
-    return (lower_bound, upper_bound)
+    return (k_lo / n_test, k_hi / n_test)
 
 
 def prediction_bounds(
@@ -398,11 +436,15 @@ def prediction_bounds(
     Parameters
     ----------
     k_cal : int
-        Number of successes in calibration data
+        Number of successes in calibration data for a **single well-defined Bernoulli event**.
+        Must be the count of a binary indicator (e.g., Z_i = 1{event}) across all n_cal trials.
     n_cal : int
-        Total number of samples in calibration data
+        Total number of trials in calibration data for the **same Bernoulli event**.
+        This is the fixed denominator (total sample size or conditional subpopulation size).
     n_test : int
-        Expected size of future test sets
+        Expected number of future trials for the **same Bernoulli event**.
+        For joint rates, this is the planned test size (fixed).
+        For conditional rates, this is an estimated future conditional subpopulation size.
     confidence : float, default=0.95
         Confidence level (e.g., 0.95 for 95% prediction bounds)
     method : str, default="simple"
@@ -435,9 +477,12 @@ def prediction_bounds(
     - Faster computation
 
     **Beta-Binomial method**:
-    - Uses Beta-Binomial distribution for exact modeling
+    - Uses Beta-Binomial distribution for exact finite-sample modeling
     - More accurate for small sample sizes
     - Slower computation
+    - Uses uniform prior Beta(1,1) and equal-tailed intervals by default
+    - For advanced use (Jeffreys prior or HPD intervals), call
+      prediction_bounds_beta_binomial() directly
 
     For large n_test, bounds approach calibration-only bounds.
     For small n_test, bounds are wider due to additional test set sampling uncertainty.
@@ -446,8 +491,11 @@ def prediction_bounds(
     applying fixed thresholds to future test sets.
     """
     if method == "simple":
-        lower = prediction_bounds_lower(k_cal, n_cal, n_test, confidence)
-        upper = prediction_bounds_upper(k_cal, n_cal, n_test, confidence)
+        raw_lower = prediction_bounds_lower(k_cal, n_cal, n_test, confidence)
+        raw_upper = prediction_bounds_upper(k_cal, n_cal, n_test, confidence)
+        # Ensure monotonicity (clip in case of numerical noise)
+        lower = min(raw_lower, raw_upper)
+        upper = max(raw_lower, raw_upper)
         return (lower, upper)
     elif method == "beta_binomial":
         return prediction_bounds_beta_binomial(k_cal, n_cal, n_test, confidence)
@@ -461,6 +509,10 @@ def ensure_ci(d: dict[str, Any] | Any, count: int, total: int, confidence: float
     If the dictionary already contains rate/CI information, use it.
     Otherwise, compute Clopper-Pearson CI from count/total.
 
+    This function re-normalizes to the requested confidence level if the
+    provided dictionary is missing bounds or if the provided bounds look
+    degenerate (NaN values).
+
     Parameters
     ----------
     d : dict
@@ -477,26 +529,28 @@ def ensure_ci(d: dict[str, Any] | Any, count: int, total: int, confidence: float
     tuple of (rate, lower, upper)
         Rate and confidence interval bounds
     """
-    # Try to get existing rate
-    r = None
+    # Initialize with NaN to detect missing values
+    r = np.nan
+    lo = np.nan
+    hi = np.nan
+
     if isinstance(d, dict):
         if "rate" in d:
             r = float(d["rate"])
         elif "proportion" in d:
             r = float(d["proportion"])
 
-    # Try to get existing CI
-    lo, hi = 0.0, 0.0
-    if isinstance(d, dict):
         if "ci_95" in d and isinstance(d["ci_95"], tuple | list) and len(d["ci_95"]) == 2:
-            lo, hi = float(d["ci_95"][0]), float(d["ci_95"][1])
-        else:
-            lo = float(d.get("lower", 0.0))
-            hi = float(d.get("upper", 0.0))
+            lo, hi = map(float, d["ci_95"])
 
-    # If missing or invalid, compute CP interval
-    if r is None or (lo == 0.0 and hi == 0.0 and (count > 0 or total > 0)):
+        else:
+            lo = float(d.get("lower", np.nan))
+            hi = float(d.get("upper", np.nan))
+
+    # If missing or invalid (NaN), compute CP interval
+    need_ci = np.isnan(r) or np.isnan(lo) or np.isnan(hi)
+    if need_ci:
         ci = cp_interval(count, total, confidence)
         return ci["proportion"], ci["lower"], ci["upper"]
 
-    return float(r), float(lo), float(hi)
+    return r, lo, hi
